@@ -43,6 +43,7 @@
 #include <gtk/gtkdnd.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkfilesel.h>
+#include <gtk/gtkfilechooser.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmain.h>
@@ -67,6 +68,8 @@
 #include <unistd.h>
 #include <string.h>
 
+#include <glib/gstdio.h>
+
 struct _GnomeIconEntryPrivate {
 	GtkWidget *fentry;
 	char *picked_file;
@@ -213,7 +216,6 @@ gnome_icon_entry_class_init (GnomeIconEntryClass *class)
 						   "or query any of its properties."),
 						 GTK_TYPE_DIALOG,
 						 G_PARAM_READABLE));
-
 }
 
 /* set_property handler for the icon entry */
@@ -388,7 +390,7 @@ entry_activated(GtkWidget *widget, GnomeIconEntry *ientry)
 	if (!filename)
 		return;
 
-	stat (filename, &buf);
+	g_stat (filename, &buf);
 	if (S_ISDIR (buf.st_mode)) {
 		gnome_icon_selection_clear (GNOME_ICON_SELECTION (priv->icon_sel), TRUE);
 		gnome_icon_selection_add_directory (GNOME_ICON_SELECTION (priv->icon_sel), filename);
@@ -412,14 +414,14 @@ setup_preview(GtkWidget *widget)
         GdkPixbuf *pixbuf, *scaled;
 	int w,h;
 	GtkWidget *frame;
-	GtkFileSelection *fs;
+	GtkFileChooser *fc;
 
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 
-	frame = g_object_get_data(G_OBJECT(widget),"frame");
-	fs = g_object_get_data(G_OBJECT(frame),"fs");
-
+	fc = GTK_FILE_CHOOSER (widget);
+	frame = gtk_file_chooser_get_preview_widget (fc);
+	
 	if((l = gtk_container_get_children(GTK_CONTAINER(frame))) != NULL) {
 		pp = l->data;
 		g_list_free(l);
@@ -428,10 +430,11 @@ setup_preview(GtkWidget *widget)
 	if(pp)
 		gtk_widget_destroy(pp);
 
-	p = gtk_file_selection_get_filename(fs);
+	p = gtk_file_chooser_get_filename(fc);
 	if(!p || !g_file_test (p,G_FILE_TEST_IS_SYMLINK | G_FILE_TEST_IS_REGULAR) ||
 	   !(pixbuf = gdk_pixbuf_new_from_file (p, NULL)))
 		return;
+	g_free (p);
 
 	w = gdk_pixbuf_get_width(pixbuf);
 	h = gdk_pixbuf_get_height(pixbuf);
@@ -511,9 +514,9 @@ static void
 browse_clicked(GnomeFileEntry *fentry, GnomeIconEntry *ientry)
 {
 	GtkWidget *w;
-	GtkWidget *hbox;
-	GtkFileSelection *fs;
+	GtkFileChooser *fc;
 	GClosure *closure;
+	gchar *path;
 
 	g_return_if_fail (fentry != NULL);
 	g_return_if_fail (GNOME_IS_FILE_ENTRY (fentry));
@@ -522,33 +525,27 @@ browse_clicked(GnomeFileEntry *fentry, GnomeIconEntry *ientry)
 
 	if(!fentry->fsw)
 		return;
-	fs = GTK_FILE_SELECTION(fentry->fsw);
+	
+	fc = GTK_FILE_CHOOSER (fentry->fsw);
 
-	hbox = fs->file_list;
-	g_return_if_fail (g_object_get_data(G_OBJECT(hbox),"frame") == NULL);
+	g_return_if_fail (gtk_file_chooser_get_preview_widget (fc) == NULL);
 
-	do {
-		hbox = hbox->parent;
-		if(!hbox) {
-			g_warning(_("Can't find an hbox, using a normal file "
-				    "selection"));
-			return;
-		}
-	} while(!GTK_IS_HBOX(hbox));
+	w = gtk_frame_new("");
+	gtk_frame_set_shadow_type (GTK_FRAME (w), GTK_SHADOW_NONE);
 
-	w = gtk_frame_new(_("Preview"));
-	gtk_widget_show(w);
-	gtk_box_pack_end(GTK_BOX(hbox),w,FALSE,FALSE,0);
-	gtk_widget_set_size_request (w,110,110);
-	g_object_set_data(G_OBJECT(w),"fs",fs);
-
-	g_object_set_data(G_OBJECT(fs->file_list),"frame",w);
-	g_object_set_data(G_OBJECT(fs->selection_entry),"frame",w);
+	gtk_file_chooser_set_preview_widget (fc, w);
+	gtk_widget_set_size_request (w, 110, 110);
 
 	closure = g_cclosure_new (G_CALLBACK (setup_preview), NULL, NULL);
-	g_object_watch_closure (G_OBJECT (fs), closure);
-	g_signal_connect_closure (fs->selection_entry, "changed",
+	g_object_watch_closure (G_OBJECT (fc), closure);
+	g_signal_connect_closure (fc, "update-preview",
 				  closure, FALSE);
+
+	/* the path can be already set */
+	if ((path = gtk_file_chooser_get_filename (fc)) != NULL)
+		setup_preview (GTK_WIDGET(fc));
+
+	g_free (path);
 }
 
 static void
@@ -567,6 +564,7 @@ icon_selected_cb (GnomeIconEntry * ientry)
 	icon = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY (ientry->_priv->fentry), FALSE);
 	
 	if (icon != NULL) {
+		g_free (priv->picked_file);
 		priv->picked_file = icon;
 		update_icon (ientry);
 		g_signal_emit (ientry, gnome_ientry_signals[CHANGED_SIGNAL], 0);
@@ -609,6 +607,7 @@ gil_icon_selected_cb(GnomeIconList *gil, gint num, GdkEvent *event, GnomeIconEnt
 	    event->type == GDK_2BUTTON_PRESS && ((GdkEventButton *)event)->button == 1) {
 		gnome_icon_selection_stop_loading
 			(GNOME_ICON_SELECTION (priv->icon_sel));
+		g_free (priv->picked_file);
 		priv->picked_file = icon;
 		icon = NULL;
 		update_icon (ientry);
@@ -824,7 +823,7 @@ drag_data_received (GtkWidget        *widget,
 	GnomeVFSFileInfo *file_info;
 	int i;
 
-	uris = g_strsplit (selection_data->data, "\r\n", -1);
+	uris = g_strsplit ((char *)selection_data->data, "\r\n", -1);
 	if (uris == NULL) {
 		gtk_drag_finish (context, FALSE, FALSE, time);
 		return;
@@ -958,7 +957,7 @@ drag_data_get  (GtkWidget          *widget,
 	g_free(file);
 	gtk_selection_data_set (selection_data,
 				selection_data->target,
-				8, string, strlen(string)+1);
+				8, (unsigned char *)string, strlen(string)+1);
 	g_free(string);
 }
 
@@ -969,6 +968,7 @@ gnome_icon_entry_instance_init (GnomeIconEntry *ientry)
 	GtkWidget *w;
 	gchar *p;
 	GClosure *closure;
+	GValue value = {0, };
 
 	ientry->_priv = g_new0(GnomeIconEntryPrivate, 1);
 
@@ -1003,13 +1003,11 @@ gnome_icon_entry_instance_init (GnomeIconEntry *ientry)
 	g_signal_connect (ientry->_priv->pickbutton, "clicked",
 			  G_CALLBACK (show_icon_selection),
 			  ientry);
-	/*FIXME: 60x60 is just larger then default 48x48, though icon sizes
-	  are supposed to be selectable I guess*/
-	gtk_widget_set_size_request (ientry->_priv->pickbutton, 60, 60);
 	gtk_container_add (GTK_CONTAINER (w), ientry->_priv->pickbutton);
 	gtk_widget_show (ientry->_priv->pickbutton);
 
 	ientry->_priv->fentry = gnome_file_entry_new (NULL, _("Browse"));
+	gnome_file_entry_set_directory_entry (GNOME_FILE_ENTRY (ientry->_priv->fentry), TRUE);
 	_add_atk_name_desc (ientry->_priv->fentry,
 			    _("Icon path"),
 			    _("Here you should enter the name of the directory "
@@ -1019,6 +1017,12 @@ gnome_icon_entry_instance_init (GnomeIconEntry *ientry)
 				G_CALLBACK (browse_clicked),
 				ientry);
 
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, TRUE);
+
+	g_object_set_property (G_OBJECT (ientry->_priv->fentry),
+			       "use_filechooser", &value);
+
 	gtk_widget_show (ientry->_priv->fentry);
 
 	p = gnome_program_locate_file (NULL /* program */,
